#include <WiFi101.h>
#include <ArduinoJson.h>

// Your wifi network login information
char NETWORK_SSID[] = "YOUR_LOCAL_WIFI_NETWORK_SSID";
char NETWORK_PASSWORD[] = "YOUR_LOCAL_WIFI_NETWORK_PASSWORD";

// The weather service server address and port
char SERVER_ADDRESS[] = "api.openweathermap.org";
int SERVER_PORT = 80;

// The weather service api key (note: you'll need to procure one from api.openweathermap.org)
String API_KEY = "YOUR_API_KEY";

// The location on earth where you want to request weather for
String LOCATION = "Mountain View, CA";

// The weather event that when detected will trigger the fan
// Choose from: Thunderstorm, Drizzle, Rain, Snow, Mist, Smoke, Haze, Dust, Fog, Sand, Ash, Squall, Tornado, Clear, Clouds 
// see https://openweathermap.org/weather-conditions for more info
String ALERT_TRIGGER = "Rain";

// The delay between queries to the weather service
int DELAY_BETWEEN_WEATHER_QUERIES = 5000;

// The pin number of the fan and how long it should run when a weather event triggers it
int FAN_PIN = 5;
int FAN_ON_DURATION = 5000;

int status = WL_IDLE_STATUS;
WiFiClient wifiClient;

// Get everything ready
void setup() {
  initSerial();
  initWifi();
  initFan();
}

// Initialize serial communication so we can print debug statements
void initSerial() {
  Serial.begin(9600);
  while (!Serial) {
    yield();
  }
  Serial.println("Serial is READY!");
}

// initialize the wifi connection to the local network
void initWifi() {
  Serial.println("Initializing Wifi...");

  // check for the presence of the shield:
  if (WiFi.status() == WL_NO_SHIELD) {
    Serial.println("WiFi shield not present, cannot initialize Wifi");
    return;
  }

  // keep trying to connect to the network until we have made a connection
  while ( status != WL_CONNECTED) {
    Serial.println("Trying to connect to SSID " + String(NETWORK_SSID) + "...");
    status = WiFi.begin(NETWORK_SSID, NETWORK_PASSWORD);
  }
  Serial.println("Wifi connected!");

  // print out the local IP address of the arduino
  IPAddress ipAddress = WiFi.localIP();
  Serial.println("Arduino IP address is ");
  Serial.println(ipAddress);
}

// initialize the fan pin so we can turn the fan on and off
void initFan() {
  pinMode(FAN_PIN, OUTPUT);
}

// the main loop of the program that runs indefinitely
void loop() {
  String response = queryWeather();
  // if we got a non-empty response
  if (response != "") {
    // do something
    reactToWeatherResponse(response);
  }

  // wait a bit until we repeat the cycle
  delay(DELAY_BETWEEN_WEATHER_QUERIES);
}

// ask the weather service for a forecast
String queryWeather() {

  String response = "";

  // if the wifi client is not connected
  if (wifiClient.connected() == 0) {
    // connect to the weather service
    Serial.println("Connecting to weather server...");
    if (wifiClient.connect(SERVER_ADDRESS, SERVER_PORT)) {
      Serial.println("connected to weather server!");
    } else {
      Serial.println("Couldn't connect to weather server");
    }
  }

  // assemble the HTTP GET request
  Serial.println("Querying weather...");
  wifiClient.print("GET /data/2.5/forecast?");
  wifiClient.print("q=" + LOCATION);
  wifiClient.print("&cnt=1");
  //wifiClient.print("&units=metric");
  wifiClient.print("&units=imperial");
  wifiClient.println("&appid=" + API_KEY);
  wifiClient.println("Host: api.openweathermap.org");
  wifiClient.println("Connection: close");
  wifiClient.println();

  // harvest the response from the server
  String line = "";
  while (wifiClient.connected()) {
    line = wifiClient.readStringUntil('\n');
    response = response + line;
  }
  Serial.println(response);

  return response;
}

// Do something given that we got a response from the weather server
void reactToWeatherResponse(String response) {
  // convert the response to a JSON object
  int responseArrayLength = response.length() + 1;
  char responseArray[responseArrayLength];
  response.toCharArray(responseArray, responseArrayLength);
  DynamicJsonDocument dynamicJsonDocument(1024);
  deserializeJson(dynamicJsonDocument, responseArray);
  // extract the forecast that will happen soonest (it's the only one in the list!)
  const char* soonestWeatherPeriodForecast = dynamicJsonDocument["list"][0]["weather"][0]["main"];
  // if the extracted forecast matches our weather trigger
  if (String(soonestWeatherPeriodForecast) == ALERT_TRIGGER) {
    Serial.println("ALERT TRIGGER DETECTED!!");
    respondToAlertTrigger();
  }
}

// do something given that our weather trigger was detected in the weather server response
void respondToAlertTrigger() {
  Serial.println("Responding to alert trigger...");
  brieflyTurnOnFan();
}

// turn on the fan for a short duration and then turn it off
void brieflyTurnOnFan() {
  Serial.println("Briefly turning on fan...");
  digitalWrite(FAN_PIN, HIGH);
  delay(FAN_ON_DURATION);
  digitalWrite(FAN_PIN, LOW);
}
